// @ts-check

const path = require("path");
const fs = require("fs");
const grabHostNames = require("../package-shared/utils/grab-host-names");
const apiPost = require("../package-shared/functions/api/query/post");

/**
 * # Make a post request to Datasquirel API
 * @async
 *
 * @param {Object} params - Single object passed
 * @param {string} [params.key] - FULL ACCESS API Key
 * @param {string} [params.database] - Database Name
 * @param {import("../package-shared/types").PostDataPayload | string} params.query - SQL query String or Request Object
 * @param {any[]} [params.queryValues] - Query Values if using "?" placeholders
 * @param {string} [params.tableName] - Name of the table to query
 * @param {boolean} [params.useLocal] - Whether to use a remote database instead of API
 * @param {boolean} [params.user_id] - User ID
 *
 * @returns { Promise<import("../package-shared/types").PostReturn> } - Return Object
 */
async function post({
    key,
    query,
    queryValues,
    database,
    tableName,
    useLocal,
    user_id,
}) {
    const grabedHostNames = grabHostNames();
    const { host, port, scheme } = grabedHostNames;

    /**
     * Check for local DB settings
     *
     * @description Look for local db settings in `.env` file and by pass the http request if available
     */
    const { DSQL_DB_HOST, DSQL_DB_USERNAME, DSQL_DB_PASSWORD, DSQL_DB_NAME } =
        process.env;

    if (
        DSQL_DB_HOST?.match(/./) &&
        DSQL_DB_USERNAME?.match(/./) &&
        DSQL_DB_PASSWORD?.match(/./) &&
        DSQL_DB_NAME?.match(/./) &&
        useLocal
    ) {
        /** @type {import("../package-shared/types").DSQL_DatabaseSchemaType | undefined} */
        let dbSchema;

        try {
            const localDbSchemaPath = path.resolve(
                process.cwd(),
                "dsql.schema.json"
            );
            dbSchema = JSON.parse(fs.readFileSync(localDbSchemaPath, "utf8"));
        } catch (error) {}

        return await apiPost({
            dbFullName: DSQL_DB_NAME,
            query,
            dbSchema,
            queryValues,
            tableName,
            useLocal,
        });
    }

    /**
     * Make https request
     *
     * @description make a request to datasquirel.com
     */
    const httpResponse = await new Promise((resolve, reject) => {
        const reqPayloadString = JSON.stringify({
            query,
            queryValues,
            database: process.env.DSQL_API_DB_NAME || database,
            tableName: tableName ? tableName : null,
        }).replace(/\n|\r|\n\r/gm, "");

        try {
            JSON.parse(reqPayloadString);
        } catch (error) {
            console.log(error);
            console.log(reqPayloadString);

            return {
                success: false,
                payload: null,
                error: "Query object is invalid. Please Check query data values",
            };
        }

        const reqPayload = reqPayloadString;

        const requPath = `/api/query/${
            user_id || grabedHostNames.user_id
        }/post`;

        const httpsRequest = scheme.request(
            {
                method: "POST",
                headers: {
                    "Content-Type": "application/json",
                    "Content-Length": Buffer.from(reqPayload).length,
                    Authorization:
                        key ||
                        process.env.DSQL_FULL_ACCESS_API_KEY ||
                        process.env.DSQL_API_KEY,
                },
                port,
                hostname: host,
                path: requPath,
            },

            /**
             * Callback Function
             *
             * @description https request callback
             */
            (response) => {
                var str = "";

                response.on("data", function (chunk) {
                    str += chunk;
                });

                response.on("end", function () {
                    try {
                        resolve(JSON.parse(str));
                    } catch (/** @type {any} */ error) {
                        console.log("Route ERROR:", error.message);

                        resolve({
                            success: false,
                            payload: null,
                            error: error.message,
                            errPayload: str,
                        });
                    }
                });

                response.on("error", (err) => {
                    resolve({
                        success: false,
                        payload: null,
                        error: err.message,
                    });
                });
            }
        );

        httpsRequest.write(reqPayload);

        httpsRequest.on("error", (error) => {
            console.log("HTTPS request ERROR =>", error);
        });

        httpsRequest.end();
    });

    /** ********************************************** */
    /** ********************************************** */
    /** ********************************************** */

    return httpResponse;
}

/** ********************************************** */
/** ********************************************** */
/** ********************************************** */

module.exports = post;
